%{
/*
 * Copyright (c) 2013 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include <stdio.h>
#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_PORT_GRP_PARSER_L_L
#include <opensm/osm_port_group.h>
#define YYSTYPE char *
#include "osm_port_grp_parser_y.h"

#define	set_yylval()								\
	free(__port_grp_parser_lval);						\
	__port_grp_parser_lval = strdup(trim_whitespace(yytext))

extern YYSTYPE __port_grp_parser_lval;
extern char *trim_whitespace(char *str);

%}

NUMBER		[1-9]+[0-9]*
HYPHEN		[-]
NL		[\n]
WS		[ \t]+

%x value
%x regexp
%x colon
%x port1
%x hyphen
%x port2

%option nounput noinput noyywrap

%%

<INITIAL>{
	[#].*\n				{ /* ignore comments */                               }
	[ \t]*\n			{ /* ignore end of line and empty lines */            }
	[ \t]*port-group[ \t]*		{                                 return P_GRP_START; }
	[ \t]*end-port-group[ \t]*	{                                 return P_GRP_END;   }
	[ \t]*use:			{                BEGIN(value);    return USE;         }
	[ \t]*name:			{                BEGIN(value);    return NAME;        }
	[ \t]*port-guid:		{                BEGIN(value);    return GUID_LIST;   }
	[ \t]*port-name:		{                BEGIN(value);    return HOSTNAME;    }
	[ \t]*port-guid-range:		{                BEGIN(value);    return RANGE;       }
	[ \t]*union-rule:		{                BEGIN(value);    return UNION;       }
	[ \t]*subtract-rule:		{                BEGIN(value);    return SUBTRACT;    }
	[ \t]*port-regexp:		{                BEGIN(regexp);   return REGEX;       }
}

<value>{
	[^#\n]*				{ set_yylval();  BEGIN(INITIAL);  return WORD;        }
}

<regexp>{
	[^:#\n]*			{ set_yylval();  BEGIN(colon);    return PATTERN;     }
}

<colon>{
	[:]				{                BEGIN(port1);    return COLON;       }
	{NL}				{                BEGIN(INITIAL);                      }
	{WS}				{                                                     }
	.				{                                                     }
}

<port1>{
	{NUMBER}/{HYPHEN}		{ set_yylval();  BEGIN(hyphen);   return PORT1;       }
	{NUMBER}			{ set_yylval();  BEGIN(INITIAL);  return PORT;        }
	{NL}				{                BEGIN(INITIAL);                      }
	{WS}				{                                                     }
}

<hyphen>{
	[-]				{                BEGIN(port2);    return HYPHEN;      }
	{NL}				{                BEGIN(INITIAL);                      }
	{WS}				{                                                     }
}

<port2>{
	{NUMBER}			{ set_yylval();                   return PORT2;       }
	{NL}				{                BEGIN(INITIAL);                      }
	{WS}				{                                                     }
}

<*>.					{                                 return ERROR;       }

%%
